-- pbtolua.lua 
-- by Florian 'FloHimself' Schfer - FloHimself@web.de
---
-- this script helps generating wrapper functions from Purebasics
-- "Declare" statement, that can be exported to Lua. (see example!)
-- usage: lua pbtolua.lua <functions.pb >tolua_functions.pb

local function parsePbCode(T)
	local isTypeTable = { 
	["b"] = "lua_isnumber",
	["w"] = "lua_isnumber",
	["l"] = "lua_isnumber", 
	["f"] = "lua_isnumber",
	["s"] = "lua_isstring"
	}
	
	local toTypeTable = { 
	["b"] = "lua_tonumber",
	["w"] = "lua_tonumber",
	["l"] = "lua_tonumber", 
	["f"] = "lua_tonumber",
	["s"] = "lua_tostring"
	}
	
	local pushTypeTable = { 
	["b"] = "lua_pushnumber",
	["w"] = "lua_pushnumber",
	["l"] = "lua_pushnumber", 
	["f"] = "lua_pushnumber",
	["s"] = "lua_pushstring"
	}
	
	local stringOfTypeTable = { 
	["b"] = "number",
	["w"] = "number",
	["l"] = "number", 
	["f"] = "number",
	["s"] = "string"
	}

	string.gsub(T, "%s*[Dd]eclare(\.[bwlfs])%s+([%w_-]+)%s*[(](.*)[)].*", function(x,y,z)
	
		local pCount = 1
		local funcType = x
		local func = y
		local funcParams = z
		
		io.write("Procedure tolua_"..func.."(L.l)\n")
		
		string.gsub(funcParams, "([%w_-]+.%w),*", function(x)

			io.write("  If "..isTypeTable[string.sub(x, -1, -1)].."(L,"..tostring(pCount)..")=0\n")
			io.write("    lua_pushstring(L, \"incorrect type of argument #"..tostring(pCount).." ("..stringOfTypeTable[string.sub(x, -1, -1)].." expected) calling function \'"..func.."\'\")\n") 
			io.write("    lua_error(L)\n") 
			io.write("  EndIf\n") 
			if string.sub(x, -1, -1) == "s" then
				io.write("  param"..tostring(pCount).."."..string.sub(x, -1, -1).."=PeekS("..toTypeTable[string.sub(x, -1, -1)].."(L,"..tostring(pCount).."))\n\n")
			else		
				io.write("  param"..tostring(pCount).."."..string.sub(x, -1, -1).."="..toTypeTable[string.sub(x, -1, -1)].."(L,"..tostring(pCount)..")\n\n")
			end
			pCount=pCount+1 
			end
		)
		
		io.write("  "..pushTypeTable[string.sub(funcType, -1, -1)].."(L,"..func.."(")
		for i=1, pCount-1 do
			io.write("param"..tostring(i))
			if i < pCount-1 then
				io.write(",")
			end
		end
		io.write("))".."\n")
		io.write("ProcedureReturn 1\n")
		io.write("EndProcedure\n\n")
		pCount = 1
		end
	)
	return 0
end

io.write("; This is automatically generated with pbtolua.lua\n")
for T in io.lines() do
	parsePbCode(T)
end 